import simple_websocket
from ..live_execution import LiveExecution
from ...contract import dashes_contract
import typing


class DashExecution(LiveExecution):
    def run(self):
        from .dash_runtime import DashRuntime

        msg_handler = DashRuntime(self, dash_json=self.runtime_json)  # type: ignore
        self.send(dashes_contract.ExecutionIdMessage(self.id))
        while True:
            try:
                type, data = self.recv()
                msg_handler.handle(type, data)
            except simple_websocket.ConnectionError:
                print("connection closed")



from .PasswordInput import PasswordInput
from .DropdownInput import DropdownInput
from .PlotlyOutput import PlotlyOutput
from .TimeInput import TimeInput
from .ChecklistInput import ChecklistInput
from .CodeInput import CodeInput
from .ImageInput import ImageInput
from .CurrencyInput import CurrencyInput
from .PandasOutput import PandasOutput
from .KanbanBoardInput import KanbanBoardInput
from .TextOutput import TextOutput
from .NumberInput import NumberInput
from .FileInput import FileInput
from .LinkOutput import LinkOutput
from .NumberSliderInput import NumberSliderInput
from .PandasRowSelectionInput import PandasRowSelectionInput
from .LatexOutput import LatexOutput
from .TextInput import TextInput
from .DateInput import DateInput
from .FileOutput import FileOutput
from .ListInput import ListInput
from .IframeOutput import IframeOutput
from .ProgressOutput import ProgressOutput
from .RatingInput import RatingInput
from .NpsInput import NpsInput
from .MarkdownOutput import MarkdownOutput
from .CnpjInput import CnpjInput
from .CardsInput import CardsInput
from .RichTextInput import RichTextInput
from .ToggleInput import ToggleInput
from .CheckboxInput import CheckboxInput
from .ImageOutput import ImageOutput
from .VideoInput import VideoInput
from .PhoneInput import PhoneInput
from .CpfInput import CpfInput
from .TextareaInput import TextareaInput
from .EmailInput import EmailInput
from .TagInput import TagInput
from .MultipleChoiceInput import MultipleChoiceInput
from .ClickInput import ClickInput
from .HtmlOutput import HtmlOutput
from .CustomInput import CustomInput



import abstra.workflows as aw
from datetime import datetime

"""
Abstra jobs are the simplest way to build repetitive tasks for your workflows. Just schedule a job and let Abstra do the rest.
"""

print("Hello world!")

# You can save and get information from the workflow context
stage = aw.get_stage()
stage["time"] = datetime.now()



from __future__ import annotations
import traceback
from ...server.api.classes import SlotJSON


class PythonProgram:
    root: SlotJSON
    state: dict

    def __init__(self, code: str) -> None:
        self.code = code

        # state: { [variable: string]: value }
        self.state = {}

    def ex(self, cmd: str):
        exec(cmd, self.state, self.state)

    def ev(self, expr: str):
        return eval(expr, self.state, self.state)

    def execute_initial_code(self):
        if not self.code:
            return

        self.ex(self.code)

    def set_variable(self, variable: str, value):
        try:
            self.state.update({"__temp_value__": value})
            self.ex(f"{variable} = __temp_value__")
        except Exception as e:
            traceback.print_exc()
        finally:
            self.state.pop("__temp_value__", None)



from dataclasses import dataclass
import typing
from .common import (  # exported
    Message,
    StdioMessage,
    AuthValidJWTMessage,
    AuthInvalidJWTMessage,
    AuthRequireInfoMessage,
)

exit_status = {
    "success": "SUCCESS",
    "lock_acquisition_failed": "LOCK_ACQUISITION_FAILED",
    "generic_exception": "GENERIC_EXCEPTION",
}


@dataclass
class CloseDTO:
    exit_status: str
    exception: typing.Union[str, None] = None

    def __post_init__(self):
        if self.exit_status not in exit_status.values():
            raise ValueError(
                f"exit_status must be one of {exit_status.values()}, got {self.exit_status}"
            )


class ExecutionIdMessage(Message):
    type = "execution-id"

    def __init__(self, execution_id: str):
        super().__init__({"executionId": execution_id})


class CloseMessage(Message):
    type = "program:end"

    def __init__(self, close_dto: CloseDTO):
        super().__init__(
            {"exitStatus": close_dto.exit_status, "exception": close_dto.exception}
        )


class FormUpdateMessage(Message):
    type = "form-update"

    def __init__(self, widgets: typing.List, validation: typing.Dict, event_seq: int):
        super().__init__(
            {
                "widgets": widgets,
                "validation": validation,
                "seq": event_seq,
            }
        )


class FormMessage(Message):
    type = "form"

    def __init__(
        self,
        widgets: typing.List,
        actions: typing.List,
        end_program: bool,
        reactive_polling_interval: int,
        steps: typing.List,
    ):
        super().__init__(
            {
                "widgets": widgets,
                "actions": actions,
                "endProgram": end_program,
                "reactivePollingInterval": reactive_polling_interval,
                "steps": steps,
            }
        )



from . import dashes as dashes_contract  # exported
from . import forms as forms_contract  # exported
from . import common as common_contract  # exported
from .common import Message, StdioMessage  # exported


def should_send(msg: Message, is_preview: bool) -> bool:
    return is_preview or not isinstance(msg, StdioMessage)
